@@ -1,4 +1,4 @@
-/* $Id: common.c,v 1.87 2004/12/26 20:23:24 mipsator Exp $ */
+/* $Id: common.c,v 1.90 2005/01/24 14:21:50 mipsator Exp $ */
 
 /*
  * Credits for patches :
@@ -134,7 +134,10 @@ bool get_make_var(char *varname, char *result, size_t rsize) {
 	char	 mfn[MAXPATHLEN],
 		 varstr[TMP_BUF_LEN];
 
-	strlcpy(mfn, TMP_MK_FILE, sizeof(mfn)); /* XXX check ? */
+	if (strlcpy_b(mfn, TMP_MK_FILE, sizeof(mfn)) == false) {
+		errorf("failed to create temporary file");
+		return(false);
+	}
 
 	mfp = tmps_open(TMP_MK_FILE, "w", mfn, sizeof(mfn), strlen(MK_FILE_EXT));
 	if (mfp != NULL) {
@@ -312,7 +315,6 @@ dynary *str_to_dynary_adv(char *str, char *seplst) {
 */
 
 bool find_file_dir(dynary *da, char *fname, char *fpath, size_t fplen) {
-	FILE		*fp;
 	char		 tstr[MAXPATHLEN],
 			*path;
 	unsigned int	 i;
@@ -323,9 +325,7 @@ bool find_file_dir(dynary *da, char *fname, char *fpath, size_t fplen) {
 		strlcpy(tstr, path, sizeof(tstr)); /* no check */
 		strlcat(tstr, "/", sizeof(tstr)); /* no check */
 		if (strlcat_b(tstr, fname, sizeof(tstr)) == true) {
-			fp = fopen(tstr, "r");
-			if (fp != NULL) {
-				fclose(fp);
+			if (access(tstr, F_OK) == 0) { /* no race condition here */
 				if (strlcpy_b(fpath, path, fplen) == true) {
 					/* fpath correctly set */
 					return(true);
@@ -382,13 +382,13 @@ bool get_file_path(char *filename, char *path, char *storage, size_t size) {
 	dynary	*bplst;
 
 	/* fill dynary with path */
-	bplst= str_to_dynary(path, CHAR_LIST_SEPARATOR);
+	bplst= str_to_dynary(path, PATH_STR_DELIMITER);
 	if (bplst == NULL) {
 		errorf("failed to put a path into a dynamic array.");
 		return(false);
 	}
 
-	/* try to locate cfgtool */
+	/* try to locate binary */
 	if (find_file(bplst, filename, storage, size) == true) {
 		rval = true;
 	} else {
