@@ -1,4 +1,4 @@
-/* $Id: func.h,v 1.41 2004/12/19 15:18:37 mipsator Exp $ */
+/* $Id: func.h,v 1.42 2005/01/24 17:58:42 mipsator Exp $ */
 
 /*
  * Copyright (c) 2003-2004 Damien Couderc
@@ -168,6 +168,10 @@
 
 #define DEFINE_DEFAULT		"1"
 
+
+/* error messages */
+#define ERR_MSG_CC_CMD		"failed to build compiler command line."
+
 /* structures */
 typedef struct {
 	char	kw[CMD_LEN];
