@@ -1,4 +1,4 @@
-/* $Id: pkgconfig.c,v 1.23 2004/02/29 20:10:35 mipsator Exp $ */
+/* $Id: pkgconfig.c,v 1.30 2004/12/26 23:42:11 mipsator Exp $ */
 
 /*
  * Copyright (c) 2003-2004 Damien Couderc
@@ -33,7 +33,7 @@
  *
  */
 
-/* include it first as if it was <sys/types.h> - this will avoid errors */ 
+/* include it first as if it was <sys/types.h> - this will avoid errors */
 #include "compat/pmk_sys_types.h"
 
 #include <dirent.h>
@@ -209,7 +209,7 @@ bool scan_dir(char *dir, pkgdata *ppd) {
 	/* open directory */
 	pdir = opendir(dir);
 	if (pdir == NULL) {
-		errorf("cannot open '%s' directory : %s.", 
+		errorf("cannot open '%s' directory : %s.",
 			dir, strerror(errno));
 		return(false);
 	}
@@ -226,13 +226,23 @@ bool scan_dir(char *dir, pkgdata *ppd) {
 						&& (pstr[l-1] == 'c')) {
 
 				/* build pkg name */
-				strlcpy(buf, pstr, sizeof(buf));
+				if (strlcpy_b(buf, pstr,
+						sizeof(buf)) == false) {
+					/* XXX err msg ? */
+					closedir(pdir);
+					return(false);
+				}
 				buf[l-3] = CHAR_EOS;
 
 				/* build full path of pc file */
-				strlcpy(fpath, dir, sizeof(fpath));
-				strlcat(fpath, "/", sizeof(fpath));
-				strlcat(fpath, pstr, sizeof(fpath));
+				strlcpy(fpath, dir, sizeof(fpath)); /* no check */
+				strlcat(fpath, "/", sizeof(fpath)); /* no check */
+				if (strlcat_b(fpath, pstr,
+						sizeof(fpath)) == false) {
+					/* XXX err msg ? */
+					closedir(pdir);
+					return(false);
+				}
 
 				hash_update_dup(ppd->files, buf, fpath);
 				/* XXX detect if the package has already been detected ? */
@@ -244,6 +254,8 @@ debugf("add module '%s' with file '%s'", buf, pstr);
 		}
 	} while (pde != NULL);
 
+	closedir(pdir);
+
 	return(true);
 }
 
@@ -303,7 +315,7 @@ bool pkg_collect(char *pkglibdir, pkgdata *ppd) {
 */
 
 bool parse_keyword(pkgcell *ppc, char *kword, char *value) {
-	int	i;
+	unsigned int	i;
 
 	for (i = 0 ; i < nb_pkgkw ; i ++) {
 		if (strncmp(kword, kw_pkg[i].kw_name, sizeof(kword)) == 0) {
@@ -311,19 +323,35 @@ bool parse_keyword(pkgcell *ppc, char *kword, char *value) {
 			switch (kw_pkg[i].kw_id) {
 				case PKGCFG_KW_NAME :
 					ppc->name = strdup(value);
+					if (ppc->name == NULL) {
+						errorf(ERRMSG_MEM);
+						return(false);
+					}
 					break;
 
 				case PKGCFG_KW_DESCR :
 					ppc->descr = strdup(value);
+					if (ppc->descr == NULL) {
+						errorf(ERRMSG_MEM);
+						return(false);
+					}
 					break;
 
 				case PKGCFG_KW_VERS :
 					ppc->version = strdup(value);
+					if (ppc->version == NULL) {
+						errorf(ERRMSG_MEM);
+						return(false);
+					}
 					break;
 
 				case PKGCFG_KW_REQS :
 					if (*value != CHAR_EOS)
 						ppc->requires = strdup(value);
+						if (ppc->requires == NULL) {
+							errorf(ERRMSG_MEM);
+							return(false);
+						}
 					break;
 
 				case PKGCFG_KW_CFLGS :
@@ -343,7 +371,7 @@ bool parse_keyword(pkgcell *ppc, char *kword, char *value) {
 			}
 
 			return(true);
-		}                                
+		}
 	}
 
 	return(true);
@@ -382,9 +410,9 @@ char *process_variables(char *pstr, htable *pht) {
 					pstr++;
 					/* skip '{' */
 					pstr++;
-					pstr = parse_identifier(pstr, var, size);
+					pstr = parse_identifier(pstr, var, sizeof(var));
 					if (pstr == NULL) {
-/*						debugf("parse_idtf returned null."); XXX */
+						/* debugf("parse_idtf returned null."); */
 						return(NULL);
 					} else {
 						/* check if identifier exists */
@@ -419,7 +447,7 @@ char *process_variables(char *pstr, htable *pht) {
 					pstr++;
 					size--;
 					if (size == 0) {
-/*						debugf("overflow."); XXX */
+						/* debugf("overflow."); */
 						return(NULL);
 					}
 					bs = false;
@@ -434,7 +462,7 @@ char *process_variables(char *pstr, htable *pht) {
 	}
 
 	if (size == 0) {
-/*		debugf("overflow."); XXX */
+		/* debugf("overflow."); */
 		return(NULL);
 	}
 
@@ -487,7 +515,11 @@ pkgcell *parse_pc_file(char *pcfile) {
 debugf("keyword = '%s', value = '%s', string = '%s'", buf, pps, pstr);
 #endif
 				/* set variable with parsed data */
-				parse_keyword(ppc, buf, pps); /* XXX check ? */
+				if (parse_keyword(ppc, buf, pps) == false) {
+					errorf("cannot fill pkgcell structure (keyword).");
+					pkgcell_destroy(ppc);
+					return(NULL);
+				}
 				break;
 
 			case '=' :
@@ -500,9 +532,12 @@ debugf("keyword = '%s', value = '%s', string = '%s'", buf, pps, pstr);
 #ifdef PKGCFG_DEBUG
 debugf("variable = '%s', value = '%s', string = '%s'", buf, pps, pstr);
 #endif
-				/* store variable in hash, XXX add only ? */
-				if (hash_update_dup(ppc->variables, buf, pps) == HASH_ADD_FAIL)
-					return(false);
+				/* store variable in hash */
+				if (hash_update_dup(ppc->variables, buf, pps) == HASH_ADD_FAIL) {
+					errorf("cannot fill pkgcell structure (variables).");
+					pkgcell_destroy(ppc);
+					return(NULL);
+				}
 
 				break;
 
@@ -586,7 +621,7 @@ debugf("recursing '%s'", reqs);
 
 	pda = str_to_dynary_adv(reqs, " ,");
 	if (pda == NULL) {
-		/* XXX errof */
+		errorf("failed to process requirements in pkg_recurse().");
 		return(false);
 	}
 
@@ -596,7 +631,11 @@ debugf("recursing '%s'", reqs);
 
 		/*  check if module has been already processed */
 		if (hash_get(ppd->cells, mod) == NULL) {
-			pkg_cell_add(ppd, mod); /* XXX check */
+			if (pkg_cell_add(ppd, mod) == NULL) {
+				errorf("failed to add a cell in pkg_recurse().");
+				da_destroy(pda);
+				return(false);
+			}
 		}
 	}
 
@@ -643,7 +682,9 @@ debugf("single_append '%s', '%s'", ostr, astr);
 		if (buf == NULL)
 			return(NULL);
 
-		snprintf(buf, s, "%s %s", ostr, astr);
+		if (snprintf_b(buf, s, "%s %s", ostr, astr) == false) {
+			return(NULL);
+		}
 
 		free(ostr);
 
@@ -815,7 +856,7 @@ bool pkg_mod_exists(pkgdata *ppd, char *mod) {
 */
 
 int compare_version(char *vref, char *vcomp) {
-	bool		 exit = false;
+	bool		 bexit = false;
 	char		*sr,
 			*sc;
 	dynary		*vr,
@@ -838,7 +879,7 @@ int compare_version(char *vref, char *vcomp) {
 		return(false);
 	}
 
-	while (exit == false) {
+	while (bexit == false) {
 		/* process reference version */
 		sr = da_idx(vr, i);
 		if (sr != NULL) {
@@ -851,7 +892,7 @@ int compare_version(char *vref, char *vcomp) {
 		} else {
 			/* end of version string */
 			ref = 0;
-			exit = true;
+			bexit = true;
 		}
 
 		/* process compared version */
@@ -865,7 +906,7 @@ int compare_version(char *vref, char *vcomp) {
 		} else {
 			/* end of version string */
 			cmp = 0;
-			exit = true;
+			bexit = true;
 		}
 
 		/* compare versions */
@@ -873,7 +914,7 @@ int compare_version(char *vref, char *vcomp) {
 
 		if (delta != 0) {
 			/* not equal end of comparison */
-			exit = true;
+			bexit = true;
 		} else {
 			i++;
 		}
