@@ -1,4 +1,4 @@
-/* $Id: pmk.c,v 1.136 2004/12/26 23:42:11 mipsator Exp $ */
+/* $Id: pmk.c,v 1.137 2005/01/24 14:21:50 mipsator Exp $ */
 
 /*
  * Credits for patches :
@@ -187,10 +187,10 @@ pmkdata *pmkdata_init(void) {
 	/* init template list */
 	ppd->tlist = NULL;
 
-	/* XXX */
+	/* init autoconf file */
 	ppd->ac_file = NULL;
 
-	/* XXX */
+	/* init default language */
 	ppd->lang = NULL;
 
 	/* init on demand */
@@ -205,8 +205,6 @@ pmkdata *pmkdata_init(void) {
 	pgd : global data structure
 
         return : -
-
-        NOTE : XXX check hash_add ?
 */
 
 bool init_var(pmkdata *pgd) {
