@@ -1,4 +1,4 @@
-/* $Id: pmksetup.c,v 1.110 2004/12/27 22:58:45 xsa Exp $ */
+/* $Id: pmksetup.c,v 1.117 2005/01/29 18:29:45 mipsator Exp $ */
 
 /*
  * Copyright (c) 2003-2004 Damien Couderc
@@ -36,13 +36,16 @@
 
 
 /*#define PMKSETUP_DEBUG 1*/
+/*#define WITHOUT_FORK 1*/
 
 /* include it first as if it was <sys/types.h> - this will avoid errors */
 #include "compat/pmk_sys_types.h"
 #include <sys/utsname.h>
+#include <sys/wait.h>
 
 #include <dirent.h>
 #include <errno.h>
+#include <pwd.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -56,6 +59,8 @@
 #include "pmksetup.h"
 
 
+extern int	 optind;
+
 FILE	*sfp;			/* scratch file pointer */
 char	sfn[MAXPATHLEN];	/* scratch file name */		
 
@@ -73,204 +78,16 @@ int	nbpredef = sizeof(predef) / sizeof(hpair);
 
 
 /*
- * Main program for pmksetup(8)
- */
-int main(int argc, char *argv[]) {
-	FILE		*config;
-	bool		 process_clopts = false,
-			 cfg_backup = false;
-	char		*pstr;
-	int		 ch,
-			 error = 0;
-	prsopt		 *ppo;
-	
-	extern int	 optind;
-#ifndef errno
-	extern int	 errno;
-#endif
-
-
-#ifndef PMK_USERMODE
-	/* pmksetup(8) must be run as root */
-	if (getuid() != 0) {
-		errorf("you must be root.");
-		exit(EXIT_FAILURE);
-	}
-#endif
-
-	ht = hash_init_adv(MAX_CONF_OPT, NULL, (void (*)(void *)) prsopt_destroy, NULL);
-	if (ht == NULL) {
-		errorf("cannot create hash table.");
-		exit(EXIT_FAILURE);
-	}
-
-	/*while ((ch = getopt(argc, argv, "a:hr:u:vV")) != -1)*/
-	while ((ch = getopt(argc, argv, "hr:u:vV")) != -1)
-		switch(ch) {
-			case 'r' :
-				/* mark to be deleted in hash */
-				if (record_data(ht, optarg, PMKSTP_REC_REMV, NULL) == false) {
-					errorf("failed to record '%s'.", optarg);
-					exit(EXIT_FAILURE);
-				}
-
-				process_clopts = true;
-				break;
-
-			case 'u' :
-				/* add or update value */
-				ppo = prsopt_init();
-				if (ppo == NULL) {
-					errorf("prsopt init failed.");
-					exit(EXIT_FAILURE);
-				}
-
-				if (parse_clopt(optarg, ppo, PRS_PMKCONF_SEP) == false) {
-					errorf("bad argument for -a option: %s.", optarg);
-					exit(EXIT_FAILURE);
-				}
-
-				/* add in hash */
-				pstr = po_get_str(ppo->value);
-				if (pstr == NULL) {
-					errorf("failed to get argument for -a option");
-					exit(EXIT_FAILURE);
-				}
-
-				/* record prsopt structure directly, fast and simple */
-				ppo->opchar = PMKSTP_REC_UPDT;
-				if (hash_update(ht, ppo->key, ppo) == HASH_ADD_FAIL) {
-					prsopt_destroy(ppo);	
-					errorf("hash update failed.");	
-					exit(EXIT_FAILURE);		
-				}
-
-				/* XXX use record_data => cost much memory and cpu XXX            */
-				/*if (record_data(ht, ppo->key, PMKSTP_REC_UPDT, pstr) == false) {*/
-				/*        errorf("failed to record '%s'.", ppo->key);             */
-				/*        prsopt_destroy(ppo);                                    */
-				/*        exit(EXIT_FAILURE);                                     */
-				/*}                                                               */
-				/*prsopt_destroy(ppo);						  */
-
-				process_clopts = true;
-				break;
-
-			case 'v' :
-				fprintf(stderr, "%s\n", PREMAKE_VERSION);
-				exit(EXIT_SUCCESS);
-				break;
-
-			case 'V' :
-				if (verbose_flag == 0)
-					verbose_flag = 1;
-				break;
-
-			case '?' :
-			default :
-				usage();
-				/* NOTREACHED */
-		}
-	argc -= optind;
-	argv += optind;
-
-
-	printf("PMKSETUP version %s", PREMAKE_VERSION);
-#ifdef DEBUG
-	printf(" [SUB #%s] [SNAP #%s]", PREMAKE_SUBVER_PMKSETUP, PREMAKE_SNAP);
-#endif
-	printf("\n\n");
-
-	/* check if syconfdir exists */
-	if (dir_exists(CONFDIR) == false) {
-		verbosef("==> Creating '%s' directory.", CONFDIR);
-		if (mkdir(CONFDIR, S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) != 0) {
-			errorf("cannot create '%s' directory : %s.",
-				CONFDIR, strerror(errno));
-			exit(EXIT_FAILURE);
-		}
-	}
-
-	if (process_clopts == false) {
-		/* standard behavior, gathering data */
-		if (gather_data(ht) == false)
-			exit(EXIT_FAILURE);
-	}
-
-	if ((open_tmp_config() == false))
-		exit(EXIT_FAILURE);
-
-	/* parse configuration file */
-	config = fopen(PREMAKE_CONFIG_PATH, "r");
-	if (config != NULL) {
-		/* switch backup flag */
-		cfg_backup = true;
-
-		printf("==> Configuration file found: %s\n", PREMAKE_CONFIG_PATH);
-		if (parse_pmkconf(config, ht, PRS_PMKCONF_SEP, check_opt) == false) {
-			fclose(config);
-			errorf("parsing failed.");
-			exit(EXIT_FAILURE);
-		} else {
-			fclose(config);
-		}
-	} else {
-		printf("==> Configuration file not found.\n");
-	}
-
-	printf("==> Merging remaining data...\n");	
-	/* writing the remaining data stored in the hash */
-	write_new_data(ht);
-	/* destroying the hash once we'r done with it */	
-	hash_destroy(ht);
-
-	if (close_tmp_config() == false)
-		exit(EXIT_FAILURE);
-
-	if (cfg_backup == true) {
-		printf("==> Backing up configuration file: %s\n", PREMAKE_CONFIG_PATH_BAK);
-		if (rename(PREMAKE_CONFIG_PATH,PREMAKE_CONFIG_PATH_BAK) != 0) {
-			errorf("configuration file backup failed: %s.", strerror(errno));
-			error = -1;
-		}
-	}
-				
-	if (error == 0) {
-		/* copying the temporary config to the system one */
-		printf("==> Saving configuration file: %s\n", PREMAKE_CONFIG_PATH);
-		/* XXX if (copy_config(sfn, PREMAKE_CONFIG_PATH) == false)*/
-		if (fcopy(sfn, PREMAKE_CONFIG_PATH, PREMAKE_CONFIG_MODE) == false)
-			exit(EXIT_FAILURE);
-	}
-
-#ifdef PMKSETUP_DEBUG
-	debugf("%s has not been deleted!", sfn);
-#else
-	if (unlink(sfn) == -1) {
-		errorf("cannot remove temporary file: '%s' : %s.",
-			sfn, strerror(errno));
-		error = -1;	
-	}
-#endif	/* PMKSETUP_DEBUG */
-
-	if (error != 0) {
-		exit(EXIT_FAILURE);	
-	}
-
-	return(0);
-}
-
-/*
- *	record data
+ * Record data
  *
- *	pht : hash table
- *	key : record name
- *	opchar : operation char
+ *	pht: hash table
+ *	key: record name
+ *	opchar: operation char
  *		PMKSTP_REC_REMV => remove
  *		PMKSTP_REC_UPDT => add/update
- *	value : record data
+ *	value: record data
  *
- *	return : true on success
+ *	returns: true on success
  */
 
 bool record_data(htable *pht, char *key, char opchar, char *value) {
@@ -296,7 +113,8 @@ bool record_data(htable *pht, char *key, char opchar, char *value) {
 		default :
 			/* unknown operation */
 #ifdef PMKSETUP_DEBUG
-	debugf("unknown '%c' operator in record_data.", opchar);
+			debugf("unknown '%c' operator in record_data.",
+								opchar);
 #endif
 			return(false);
 	}
@@ -309,18 +127,29 @@ bool record_data(htable *pht, char *key, char opchar, char *value) {
 	return(true);
 }
 
-
 /*
- * gathering of system data, predefinied data, ...
+ * Gathering of system data, predefinied data, ...
+ *
+ *	pht: storage hash table
+ *
+ *	returns: true on success else false
  */
 
 bool gather_data(htable *pht) {
 	printf("==> Looking for default parameters...\n");
 
-	/* gather env variables and look for specific binaries */
-	if ((get_env_vars(pht) == false) || (get_binaries(pht) == false))
-		return(false);	/* XXX error messages ? */
+	/* gather env variables */
+	if (get_env_vars(pht) == false) {
+		errorf("failed to gather env variables.");
+		return(false);
+	}
 
+ 	/* gather env variables and look for specific binaries */
+	if (get_binaries(pht) == false) {
+		errorf("failed to locate binaries.");
+		return(false);
+	}
+ 
 	/* set predifined variables */
 	if (predef_vars(pht) == false) {
 		errorf("predefined variables.");
@@ -350,57 +179,62 @@ bool gather_data(htable *pht) {
 }
 
 /*
- * write remaining data
+ * Write remaining data
  *
  * 	pht: hash table
+ *
+ * 	returns: -
  */
-void write_new_data(htable *pht) {
+
+bool write_new_data(htable *pht) {
 	char		*val;
 	hkeys		*phk;
 	prsopt		*ppo;
 	unsigned int	 i;
 
 	phk = hash_keys(pht);
-	if (phk != NULL) {
-		/* sort hash table */
-		qsort(phk->keys, phk->nkey, sizeof(char *), keycomp);
+	if (phk == NULL) {
+		verbosef("Nothing to merge.");
+		return(true);
+	}
 
-		/* processing remaining keys */
-		for(i = 0 ; i < phk->nkey ; i++) {
+	/* sort hash table */
+	qsort(phk->keys, phk->nkey, sizeof(char *), keycomp);
+
+	/* processing remaining keys */
+	for(i = 0 ; i < phk->nkey ; i++) {
 #ifdef PMKSETUP_DEBUG
-	debugf("write_new_data: processing '%s'", phk->keys[i]);
+		debugf("write_new_data: processing '%s'", phk->keys[i]);
 #endif
-			ppo = hash_get(pht, phk->keys[i]);
-			if (ppo == NULL) {
-#ifdef PMKSETUP_DEBUG
-				debugf("write_new_data: unable to get '%s' record.", phk->keys[i]);
-#endif
-				/* XXX  exit ? */
-			} else {
-				/* check if this key is marked for being removed */
-				if (ppo->opchar != PMKSTP_REC_REMV) {
-					/* add value */
-					val = po_get_str(ppo->value);
-					if (val != NULL) {
-						fprintf(sfp, PMKSTP_WRITE_FORMAT, phk->keys[i], CHAR_ASSIGN_UPDATE, val);
-#ifdef PMKSETUP_DEBUG
-					} else {
-						debugf("write_new_data: skipping '%s', empty value.", phk->keys[i]);
-#endif
-						/* XXX  exit ? */
-					}
-#ifdef PMKSETUP_DEBUG
-				} else {
-					debugf("write_new_data: skipping '%s', remove record.", phk->keys[i]);
-#endif
-				}
-			}
+		ppo = hash_get(pht, phk->keys[i]);
+		if (ppo == NULL) {
+			errorf("unable to get '%s' record.", phk->keys[i]);
+			return(false);
 		}
 
-		hash_free_hkeys(phk);
-	} else {
-		verbosef("Nothing to merge.");
+		/* check if this key is marked for being removed */
+		if (ppo->opchar == PMKSTP_REC_REMV) {
+#ifdef PMKSETUP_DEBUG
+			debugf("write_new_data: skipping '%s', remove record.",
+								phk->keys[i]);
+#endif
+			continue;
+		}
+
+		/* add value */
+		val = po_get_str(ppo->value);
+		if (val == NULL) {
+			errorf("'%s' key has an empty value.", phk->keys[i]);
+			return(false);
+		}
+
+		fprintf(sfp, PMKSTP_WRITE_FORMAT, phk->keys[i],
+						CHAR_ASSIGN_UPDATE, val);
 	}
+
+	hash_free_hkeys(phk);
+
+	return(true);
 }
 
 
@@ -413,21 +247,22 @@ void write_new_data(htable *pht) {
  *	a: first element to compare
  *	b: second element to compare
  *
- *	returns an integer greater than, equal to, or
+ *	returns: an integer greater than, equal to, or
  *		less than 0 according as the character a is
  *		greather, equal to, or less than the character b.
  */
+
 int keycomp(const void *a, const void *b) {
-	return(strcmp(*(const char **)a, *(const char **)b));
+	return(strcmp(*(const char **) a, *(const char **) b));
 }
 
 /*
- *	check and process option
+ * Check and process option
  *
- *	pht : htable for comparison
- *	popt : option to process
+ *	pht: htable for comparison
+ *	popt: option to process
  *
- *	return : boolean
+ *	returns: boolean
  */
 
 bool check_opt(htable *cht, prsopt *popt) {
@@ -451,117 +286,65 @@ bool check_opt(htable *cht, prsopt *popt) {
 #endif
 
 	ppo = hash_get(ht, popt->key);
-	/*if ((ppo != NULL) && (ppo->value != NULL)) {*//* on remove value is NULL !!! */
-	if (ppo != NULL) {
-		/* XXX check value */
-		recval = po_get_str(ppo->value);
-
+	if (ppo == NULL) {
 #ifdef PMKSETUP_DEBUG
-		debugf("check_opt: original opchar '%c'", popt->opchar);
-#endif
-
-		/* checking the VAR<->VALUE separator */
-		switch (popt->opchar) {
-			case CHAR_ASSIGN_UPDATE :
-#ifdef PMKSETUP_DEBUG
-				debugf("check_opt: update opchar '%c'", ppo->opchar);
-#endif
-				switch (ppo->opchar) {
-					case PMKSTP_REC_REMV :
-						/* do nothing, value will not be refreshed */
-						verbosef("Removing '%s'", popt->key);
-						break;
-					case PMKSTP_REC_UPDT :
-						/* update value */
-						fprintf(sfp, PMKSTP_WRITE_FORMAT, popt->key, CHAR_ASSIGN_UPDATE, recval);
-#ifdef PMKSETUP_DEBUG
-						debugf("check_opt() appended '" PMKSTP_WRITE_FORMAT "'", popt->key, CHAR_ASSIGN_UPDATE, recval);
-#endif
-						break;
-
-				}
-				hash_delete(cht, popt->key);
-				break;
-
-			case CHAR_ASSIGN_STATIC :
-				/* static definition, stay unchanged */
-				fprintf(sfp, PMKSTP_WRITE_FORMAT, popt->key, CHAR_ASSIGN_STATIC, optval);
-#ifdef PMKSETUP_DEBUG
-				debugf("check_opt() appended '" PMKSTP_WRITE_FORMAT "'", popt->key, CHAR_ASSIGN_STATIC, optval);
-#endif
-				hash_delete(cht, popt->key);
-				break;
-
-			default :
-				/* should not happen now */
-				errorf("unknown operator '%c'.", popt->opchar);
-				return(false);
-				break;
-		}
-	} else {
-#ifdef PMKSETUP_DEBUG
-	debugf("check_opt: key '%s' does not exist.", popt->key);
+		debugf("check_opt: key '%s' does not exist.", popt->key);
 #endif
 		fprintf(sfp, PMKSTP_WRITE_FORMAT, popt->key, popt->opchar, optval);
-	}
-
-	return(true);
-}
-
-/*
- * Open temporary configuration file
- *
- *	return:  boolean
- */
-bool open_tmp_config(void) {
-	int	fd;
-
-	/* creating temporary file to build new configuration file */
-	if (snprintf_b(sfn, sizeof(sfn), PREMAKE_CONFIG_TMP) == false)
-		return(false);
-
-	fd = mkstemp(sfn);
-	if (fd == -1) {
-		errorf("could not create temporary file '%s' : %s.",
-			sfn, strerror(errno));
-		return(false);
-	}
-
-	sfp = fdopen(fd, "w");
-	if (sfp == NULL) {
-		unlink(sfn);
-		close(fd);
-		errorf("cannot open temporary file '%s' : %s.",
-			sfn, strerror(errno));
-		return(false);
-	}
-	return(true);
-}
-
-
-/*
- * Close temporary configuration file
- *
- *	it deletes the temporary file on exit except if the
- *	PMKSETUP_DEBUG mode is on.
- *
- *	return:  boolean
- */
-bool close_tmp_config(void) {
-	if (sfp == NULL)
 		return(true);
-
-	if (fclose(sfp) != 0) {
-		errorf("cannot close temporary file '%s' : %s.",
-			sfp, strerror(errno));
-		return(false);
 	}
 
-	sfp = NULL; /* XXX useful ? */
+	/* XXX check value ? */
+	recval = po_get_str(ppo->value);
+
+#ifdef PMKSETUP_DEBUG
+	debugf("check_opt: original opchar '%c'", popt->opchar);
+#endif
+
+	/* checking the VAR<->VALUE separator */
+	switch (popt->opchar) {
+		case CHAR_ASSIGN_UPDATE :
+#ifdef PMKSETUP_DEBUG
+			debugf("check_opt: update opchar '%c'", ppo->opchar);
+#endif
+			switch (ppo->opchar) {
+				case PMKSTP_REC_REMV :
+					/* do nothing, value will not be refreshed */
+					verbosef("Removing '%s'", popt->key);
+					break;
+
+				case PMKSTP_REC_UPDT :
+					/* update value */
+					fprintf(sfp, PMKSTP_WRITE_FORMAT, popt->key, CHAR_ASSIGN_UPDATE, recval);
+#ifdef PMKSETUP_DEBUG
+					debugf("check_opt() appended '" PMKSTP_WRITE_FORMAT "'", popt->key, CHAR_ASSIGN_UPDATE, recval);
+#endif
+					break;
+
+			}
+
+			hash_delete(cht, popt->key);
+			break;
+
+		case CHAR_ASSIGN_STATIC :
+			/* static definition, stay unchanged */
+			fprintf(sfp, PMKSTP_WRITE_FORMAT, popt->key, CHAR_ASSIGN_STATIC, optval);
+#ifdef PMKSETUP_DEBUG
+			debugf("check_opt() appended '" PMKSTP_WRITE_FORMAT "'", popt->key, CHAR_ASSIGN_STATIC, optval);
+#endif
+			hash_delete(cht, popt->key);
+			break;
+
+		default :
+			/* should not happen now */
+			errorf("unknown operator '%c'.", popt->opchar);
+			return(false);
+			break;
+	}
+
 	return(true);
 }
 
-
 /*
  * Get the environment variables needed for the configuration file
  *
@@ -569,6 +352,7 @@ bool close_tmp_config(void) {
  *
  *	return:  boolean
  */
+
 bool get_env_vars(htable *pht) {
 	struct utsname	 utsname;
 	char		*bin_path;
@@ -596,12 +380,6 @@ bool get_env_vars(htable *pht) {
 		return(false);
 	}
 
-	/*
-	 * replace the string separator in the PATH
-	 * to fit to our standards
-	 */
-	char_replace(bin_path, PATH_STR_DELIMITER, CHAR_LIST_SEPARATOR);
-
 	if (record_data(pht, PMKCONF_PATH_BIN, 'u', bin_path) == false)
 		return(false);
 	verbosef("Setting '%s' => '%s'", PMKCONF_PATH_BIN, bin_path);
@@ -609,14 +387,14 @@ bool get_env_vars(htable *pht) {
 	return(true);
 }
 
-
 /*
  * Look for location of some predefined binaries
  *
  *	ht: hash table where we have to store the values
  *
- *	return: boolean
+ *	returns: boolean
  */
+
 bool get_binaries(htable *pht) {
 	char		 fbin[MAXPATHLEN],	/* full binary path */
 			*pstr;
@@ -637,11 +415,11 @@ bool get_binaries(htable *pht) {
 
 	pstr = po_get_str(ppo->value);
 	if (pstr == NULL) {
-		/* XXX error */
+		errorf("unable to get binary path value.");
 		return(false);
 	}
 
-	stpath = str_to_dynary(pstr , CHAR_LIST_SEPARATOR);
+	stpath = str_to_dynary(pstr , PATH_STR_DELIMITER);
 	if (stpath == NULL) {
 		errorf("could not split the PATH environment variable correctly.");
 		return(false);	
@@ -691,6 +469,7 @@ bool get_binaries(htable *pht) {
  *
  *	returns:  boolean
  */
+
 bool predef_vars(htable *pht) {
 	int	i;
 
@@ -706,10 +485,13 @@ bool predef_vars(htable *pht) {
 }
 
 /*
+ * Check echo output
+ *
  *	pht: hash table where we have to store the values
  *
  *	returns:  boolean
  */
+
 bool check_echo(htable *pht) {
 	FILE	*echo_pipe = NULL;
 	char	buf[TMP_BUF_LEN],
@@ -783,14 +565,14 @@ bool check_echo(htable *pht) {
 
 
 /*
- *	check pkgconfig libpath
+ * Check pkgconfig libpath
  *
  *	pht: hash table where we have to store the values
  *
  *	returns:  boolean
  *
- *	NOTE: this path must be relative to the real prefix. This means
- *		that we can't rely on predefined prefix.
+ * NOTE: this path must be relative to the real prefix. This means
+ * that we can't rely on predefined prefix.
  */
 
 bool check_libpath(htable *pht) {
@@ -825,9 +607,9 @@ bool check_libpath(htable *pht) {
 }
 
 /*
- *	check the cpu family 
+ * Check the cpu family 
  *
- *	uname_m : uname machine string 
+ *	uname_m: uname machine string 
  *
  *	returns:  cpu family string or NULL
  */
@@ -842,27 +624,35 @@ bool get_cpu_data(htable *pht) {
 	prsopt		*ppo;
 	unsigned int	 i;
 
-
 	ppo = hash_get(pht, PMKCONF_OS_ARCH);
-	if (ppo == NULL)
+	if (ppo == NULL) {
+		errorf("failed to get value for %s", PMKCONF_OS_ARCH);
 		return(false);
+	}
 
 	po = ppo->value;
-	if (po == NULL)
+	if (po == NULL) {
+		errorf("unexpected data for %s", PMKCONF_OS_ARCH);
 		return(false);
+	}
 
 	uname_m = po_get_str(po);
-	if (uname_m == NULL)
+	if (uname_m == NULL) {
+		errorf("unexpected value for %s", PMKCONF_OS_ARCH);
 		return(false);
+	}
 
 	pdata = parse_cpu_data(PMKCPU_DATA);
 	if (pdata == NULL) {
+		/* error message already done */
 		return(false);
 	}
 
 	pstr = check_cpu_arch(uname_m, pdata); /* no check, never NULL */
-	if (record_data(pht, PMKCONF_HW_CPU_ARCH, 'u', pstr) == false)
+	if (record_data(pht, PMKCONF_HW_CPU_ARCH, 'u', pstr) == false) {
+		errorf("failed to record value for %s", PMKCONF_HW_CPU_ARCH);
 		return(false);
+	}
 	verbosef("Setting '%s' => '%s'", PMKCONF_HW_CPU_ARCH, pstr);
 
 	spht = arch_wrapper(pdata, pstr);
@@ -870,7 +660,7 @@ bool get_cpu_data(htable *pht) {
 		phk = hash_keys(spht);
 		if (phk != NULL) {
 			for(i = 0 ; i < phk->nkey ; i++) {
-				pstr = hash_get(spht, phk->keys[i]); /* XXX check ? */
+				pstr = hash_get(spht, phk->keys[i]); /* should not be NULL */
 
 				if (record_data(pht, phk->keys[i], 'u', pstr) == false)
 					return(false);
@@ -888,9 +678,9 @@ bool get_cpu_data(htable *pht) {
 }
 
 /*
- *	check if a directory does exist
+ * Check if a directory does exist
  *
- *	fdir : directory to search
+ *	fdir: directory to search
  *
  *	returns:  boolean
  */
@@ -912,11 +702,11 @@ bool dir_exists(const char *fdir) {
 }
 
 /*
- *	byte order check
+ * Byte order check
  *
- *	pht : hash table to store value
+ *	pht: hash table to store value
  *
- *	return : boolean value
+ *	returns: boolean value
  */
 
 bool byte_order_check(htable *pht) {
@@ -948,30 +738,14 @@ bool byte_order_check(htable *pht) {
 	return(true);
 }
 
-
-/*
- * Replace a character in a string
- *
- *	buf: string we want to modify
- *	search: the char with want to replace
- *	replace: the new char which will replace 'search'
- */
-void char_replace(char *buf, const char search, const char replace) {
-	char	c;
-	int	i = 0;
-
-	while ((c = buf[i]) != CHAR_EOS) {
-		if (c == search)
-			c = replace;
-		buf[i] = c;
-		i++;
-	}
-}
-
-
 /*
  * Simple formated verbose function
+ *
+ *	fmt: format string
+ *
+ *	returns: -
  */
+
 void verbosef(const char *fmt, ...) {
 	char	buf[MAX_ERR_MSG_LEN];
 	va_list	plst;
@@ -985,13 +759,394 @@ void verbosef(const char *fmt, ...) {
 	}
 }
 
-
 /*
  * pmksetup(8) usage
  */
+
 void usage(void) {
 	fprintf(stderr, "usage: pmksetup [-hVv] "
 		"[-r variable] [-u variable=value]\n");
 	exit(EXIT_FAILURE);
 }
 
+/*
+ * Detection loop
+ *
+ *	argc:	argument number
+ *	argv:	argument array
+ *
+ *	returns: true on success else false
+ */
+
+bool detection_loop(int argc, char *argv[]) {
+	FILE		*config;
+	bool		 process_clopts = false,
+			 cfg_backup = false;
+	char		*pstr;
+	int		 ch;
+	prsopt		 *ppo;
+	
+#ifndef errno
+	extern int	 errno;
+#endif /* errno */
+
+	optind = 1;
+	while ((ch = getopt(argc, argv, PMKSTP_OPT_STR)) != -1) {
+		switch(ch) {
+			case 'r' :
+				/* mark to be deleted in hash */
+				if (record_data(ht, optarg, PMKSTP_REC_REMV, NULL) == false) {
+					errorf("failed to record '%s'.", optarg);
+					return(false);
+				}
+
+				process_clopts = true;
+				break;
+
+			case 'u' :
+				/* add or update value */
+				ppo = prsopt_init();
+				if (ppo == NULL) {
+					errorf("prsopt init failed.");
+					return(false);
+				}
+
+				if (parse_clopt(optarg, ppo, PRS_PMKCONF_SEP) == false) {
+					errorf("bad argument for -a option: %s.", optarg);
+					return(false);
+				}
+
+				/* add in hash */
+				pstr = po_get_str(ppo->value);
+				if (pstr == NULL) {
+					errorf("failed to get argument for -a option");
+					return(false);
+				}
+
+				/* record prsopt structure directly, fast and simple */
+				ppo->opchar = PMKSTP_REC_UPDT;
+				if (hash_update(ht, ppo->key, ppo) == HASH_ADD_FAIL) {
+					prsopt_destroy(ppo);	
+					errorf("hash update failed.");	
+					return(false);
+				}
+
+				/* XXX use record_data => cost much memory and cpu XXX            */
+				/*if (record_data(ht, ppo->key, PMKSTP_REC_UPDT, pstr) == false) {*/
+				/*        errorf("failed to record '%s'.", ppo->key);             */
+				/*        prsopt_destroy(ppo);                                    */
+				/*        return(false);                                          */
+				/*}                                                               */
+				/*prsopt_destroy(ppo);						  */
+
+				process_clopts = true;
+				break;
+
+			case 'v' :
+			case 'V' :
+				/*
+				 * already processed in main()
+				 */
+				break;
+
+			case '?' :
+			default :
+				usage();
+				/* NOTREACHED */
+		}
+	}
+
+	argc -= optind;
+	argv += optind;
+
+	printf("PMKSETUP version %s", PREMAKE_VERSION);
+#ifdef DEBUG
+	printf(" [SUB #%s] [SNAP #%s]", PREMAKE_SUBVER_PMKSETUP, PREMAKE_SNAP);
+#endif /* DEBUG */
+	printf("\n\n");
+
+	if (process_clopts == false) {
+		/* standard behavior, gathering data */
+		if (gather_data(ht) == false)
+			return(false);
+	}
+
+	/* parse configuration file */
+	config = fopen(PREMAKE_CONFIG_PATH, "r");
+	if (config != NULL) {
+		/* switch backup flag */
+		cfg_backup = true;
+
+		printf("==> Configuration file found: %s\n",
+						PREMAKE_CONFIG_PATH);
+		if (parse_pmkconf(config, ht, PRS_PMKCONF_SEP,
+						check_opt) == false) {
+			fclose(config);
+			errorf("parsing failed.");
+			return(false);
+		} else {
+			fclose(config);
+		}
+	} else {
+		printf("==> Configuration file not found.\n");
+	}
+
+	printf("==> Merging remaining data...\n");
+	/* writing the remaining data stored in the hash */
+	if (write_new_data(ht) == false)
+		return(false);
+
+	return(true);
+}
+
+/*
+ * Child main loop
+ *
+ *	uid:	privsep user id
+ *	gid:	privsep user group
+ *	argc:	argument number
+ *	argv:	argument array
+ *
+ *	returns: -
+ */
+
+void child_loop(uid_t uid, gid_t gid, int argc, char *argv[]) {
+#ifndef WITHOUT_FORK
+	if (getuid() == 0) {
+		/* user has root privs, DROP THEM ! */
+		if (setgid(gid) != 0) {
+			/* failed to change gid */
+			errorf(EMSG_PRIV_FMT, "gid");
+			_exit(EXIT_FAILURE);
+		}
+
+		if (setegid(gid) != 0) {
+			/* failed to change egid */
+			errorf(EMSG_PRIV_FMT, "egid");
+			_exit(EXIT_FAILURE);
+		}
+
+		if (setuid(uid) != 0) {
+			/* failed to change uid */
+			errorf(EMSG_PRIV_FMT, "uid");
+			_exit(EXIT_FAILURE);
+		}
+
+		if (seteuid(uid) != 0) {
+			/* failed to change euid */
+			errorf(EMSG_PRIV_FMT, "euid");
+			_exit(EXIT_FAILURE);
+		}
+	}
+#endif /* WITHOUT_FORK */
+
+#ifdef PMKSETUP_DEBUG
+	debugf("gid = %d", getgid());
+	debugf("uid = %d", getuid());
+#endif /* PMKSETUP_DEBUG */
+
+	ht = hash_init_adv(MAX_CONF_OPT, NULL, (void (*)(void *)) prsopt_destroy, NULL);
+	if (ht == NULL) {
+		errorf("cannot create hash table.");
+		exit(EXIT_FAILURE);
+	}
+
+	if (detection_loop(argc, argv) == false) {
+		hash_destroy(ht);
+		exit(EXIT_FAILURE);
+	}
+
+	/* destroying the hash once we're done with it */
+	hash_destroy(ht);
+		
+}
+
+/*
+ * Parent main loop
+ *
+ *	pid: pid of the child
+ *
+ *	returns: -
+ */
+
+void parent_loop(pid_t pid) {
+	bool	error = false;
+#ifndef WITHOUT_FORK
+	int	status;
+
+	waitpid(pid, &status, WUNTRACED);
+
+#ifdef PMKSETUP_DEBUG
+	debugf("waitpid() status = %d", WEXITSTATUS(status));
+#endif /* PMKSETUP_DEBUG */
+#endif /* WITHOUT_FORK */
+
+	if (fclose(sfp) != 0) {
+		/* hazardous result => exit */
+		errorf("cannot close temporary file '%s' : %s.", sfp,
+							strerror(errno));
+		exit(EXIT_FAILURE);
+	}
+
+#ifdef WITHOUT_FORK
+	/*
+	 * without fork, the child part will exit() if a problem occur
+	 * so we shouldn't need to care about what happens
+	 */
+#else
+	/* if child status is ok, writing changes */
+	if (status == 0) {
+#endif
+		/*
+		 * check if pmk.conf already exists
+		 * NOTE: no race condition here for access(), BUT
+		 * BE CAREFUL if changes are to be made.
+		 */ 
+		if (access(PREMAKE_CONFIG_PATH, F_OK) == 0) { /* see above */
+			/* backup configuration file */
+			printf("==> Backing up configuration file: %s\n",
+						PREMAKE_CONFIG_PATH_BAK);
+
+			if (rename(PREMAKE_CONFIG_PATH,
+					PREMAKE_CONFIG_PATH_BAK) != 0) {
+				errorf("configuration file backup failed: %s.",
+							strerror(errno));
+				error = true;
+			}
+		}
+
+		/* copying the temporary config to the system one */
+		printf("==> Saving configuration file: %s\n",
+						PREMAKE_CONFIG_PATH);
+		if (fcopy(sfn, PREMAKE_CONFIG_PATH,
+					PREMAKE_CONFIG_MODE) == false) {
+			errorf("failed to copy temporary file '%s'.", sfn);
+			exit(EXIT_FAILURE);
+		}
+#ifndef WITHOUT_FORK
+	}
+#endif
+
+#ifdef PMKSETUP_DEBUG
+	debugf("%s has not been deleted!", sfn);
+#else
+	if (unlink(sfn) == -1) {
+		errorf("cannot remove temporary file: '%s' : %s.",
+					sfn, strerror(errno));
+		error = true;
+	}
+#endif	/* PMKSETUP_DEBUG */
+
+#ifndef WITHOUT_FORK
+	if (status != 0) {
+		errorf("child failed (status %d)", status);
+		exit(EXIT_FAILURE);
+	}
+#endif /* WITHOUT_FORK */
+
+	if (error == true)
+		exit(EXIT_FAILURE);
+}
+
+/*
+ * Main loop
+ */
+
+int main(int argc, char *argv[]) {
+	struct passwd	*pw;
+	gid_t		 gid = 0;
+	pid_t		 pid;
+	uid_t		 uid = 0;
+
+	int		 ch;
+
+	optind = 1;
+	while ((ch = getopt(argc, argv, PMKSTP_OPT_STR)) != -1) {
+		switch(ch) {
+			case 'r' :
+			case 'u' :
+				/*
+				 * ignore this options at this level, it
+				 * will be processed later
+				 */
+				break;
+
+			case 'v' :
+				fprintf(stderr, "%s\n", PREMAKE_VERSION);
+				return(true);
+				break;
+
+			case 'V' :
+				if (verbose_flag == 0)
+					verbose_flag = 1;
+				break;
+
+			case '?' :
+			default :
+				usage();
+				/* NOTREACHED */
+		}
+	}
+
+
+	if (getuid() == 0) {
+#ifdef PMKSETUP_DEBUG
+		debugf("PRIVSEP_USER = '%s'", PRIVSEP_USER);
+#endif /* PMKSETUP_DEBUG */
+		pw = getpwnam(PRIVSEP_USER);
+		if (pw == NULL) {
+			errorf("cannot get user data for '%s'.", PRIVSEP_USER);
+			exit(EXIT_FAILURE);
+		}
+		uid = pw->pw_uid;
+		gid = pw->pw_gid;
+	}
+
+	/* check if syconfdir exists */
+	if (access(CONFDIR, F_OK) != 0) { /* no race condition, just mkdir() */
+		verbosef("==> Creating '%s' directory.", CONFDIR);
+		if (mkdir(CONFDIR, S_IRWXU | S_IRGRP | S_IXGRP |
+						S_IROTH | S_IXOTH) != 0) {
+			errorf("cannot create '%s' directory : %s.",
+						CONFDIR, strerror(errno));
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	sfp = tmp_open(PREMAKE_CONFIG_TMP, "w", sfn, sizeof(sfn));
+	if (sfp == NULL) {
+		errorf("cannot open temporary file '%s' : %s.",
+				sfn, strerror(errno));
+		exit(EXIT_FAILURE);
+}
+
+#ifndef WITHOUT_FORK
+	/* forking detection code */
+	pid = fork();
+	switch (pid) {
+		case -1:
+			/* fork failed */
+			errorf("fork failed.");
+			break;
+
+		case 0:
+			/* child start here after forking */
+#endif /* WITHOUT_FORK */
+			child_loop(uid, gid, argc, argv);
+#ifndef WITHOUT_FORK
+			break;
+
+		default:
+			/* parent continue here after forking */
+#else
+			pid = getpid();
+#endif /* WITHOUT_FORK */
+			parent_loop(pid);
+#ifndef WITHOUT_FORK
+			break;
+	}
+#endif /* WITHOUT_FORK */
+
+	return(EXIT_SUCCESS);
+
+}
