@@ -1,4 +1,4 @@
-/* $Id: pmksetup.h,v 1.46 2004/12/25 00:58:30 xsa Exp $ */
+/* $Id: pmksetup.h,v 1.51 2005/01/29 18:50:10 mipsator Exp $ */
 
 /*
  * Copyright (c) 2003-2004 Xavier Santolaria <xavier@santolaria.net>
@@ -43,9 +43,9 @@
 
 
 /* pmksetup specific version */
-#define PREMAKE_SUBVER_PMKSETUP	"7"
+#define PREMAKE_SUBVER_PMKSETUP	"9"
 
-#define PREMAKE_CONFIG_TMP	PREMAKE_TMP_DIR "/pmk.XXXXXXXX"
+#define PREMAKE_CONFIG_TMP	CONFDIR "/pmk.conf_XXXXXXXX"
 #define PREMAKE_CONFIG_MODE	S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH
 
 
@@ -54,6 +54,12 @@
 #define DATADIR	"/DATADIR_not_defined"
 #endif
 
+/* set default priviledged user */
+#ifndef PRIVSEP_USER
+#define PRIVSEP_USER	"nobody"
+#endif
+
+
 #define PMKCPU_DATA		DATADIR "/pmkcpu.dat"
 
 /* define printf format in pmk.conf */
@@ -67,9 +73,6 @@
 
 #define MAX_CONF_OPT		256	/* XXX temp maximum hash entries */
 
-/* string delimiter of the $PATH variable */
-#define PATH_STR_DELIMITER	':'
-
 #define ECHO_CMD	"echo \"one\\c\"; echo -n two; echo three"
 #define	ECHO_EMPTY	""
 #define	ECHO_N		"-n"
@@ -77,6 +80,9 @@
 #define	ECHO_NL		"\\n"
 #define	ECHO_HT		"\\t"
 
+#define PMKSTP_OPT_STR	"hr:u:vV"	/* "a:hr:u:vV" */
+
+#define EMSG_PRIV_FMT	"Failed to change privilege (%s)"
 
 /*
  * Look for location of some predefined binaries.
@@ -121,11 +127,13 @@ bool	check_echo(htable *);
 bool	check_libpath(htable *);
 bool	get_cpu_data(htable *);
 bool	dir_exists(const char *);
-bool	byte_order_check(htable *pht);
+bool	byte_order_check(htable *);
 int	keycomp(const void *, const void *);
-void	char_replace(char *, const char, const char);
-void	write_new_data(htable *);
-void	verbosef(const char *fmt, ...);
+bool	write_new_data(htable *);
+void	verbosef(const char *, ...);
 void	usage(void);
+bool	detection_loop(int, char *[]);
+void	child_loop(uid_t, gid_t, int, char *[]);
+void	parent_loop(pid_t);
 
 #endif	/* _PMKSETUP_H_ */
