@@ -1,4 +1,4 @@
-/* $Id: premake.h,v 1.113 2004/12/11 19:40:00 mipsator Exp $ */
+/* $Id: premake.h,v 1.119 2005/01/29 18:50:10 mipsator Exp $ */
 
 /*
  * Copyright (c) 2003-2004 Damien Couderc
@@ -94,9 +94,9 @@
 /* major version */
 #define PREMAKE_VMAJOR	"0"
 /* minor version */
-#define PREMAKE_VMINOR	"8"
+#define PREMAKE_VMINOR	"9"
 /* minor subversion, only used for bugfixes  */
-#define PREMAKE_VSUB	"1"
+#define PREMAKE_VSUB	"2"
 
 /* full version */
 #ifndef PREMAKE_VSUB	
@@ -106,7 +106,7 @@
 #endif
 
 /* only used for snapshots, comment for release */
-#define PREMAKE_SNAP		"9"
+/* #define PREMAKE_SNAP		"0"		*/
 
 /* build version string */
 #ifndef PREMAKE_SNAP
@@ -232,5 +232,8 @@
 /* size for temporary buffers */
 #define TMP_BUF_LEN		512
 
+/* string delimiter PATH variables */
+#define PATH_STR_DELIMITER	':'
+
 
 #endif /* _PREMAKE_H_ */
